<?php
/*
Plugin Name: Fruit WP Functions
Plugin URI: http://favouritefruit.com/dev/fruit-wp-functions
Description: Numerous convenience functions for Wordpress - shortcodes, widgets, etc.
Version: 1.0
Author: Favourite Fruit
Author URI: http://favouritefruit.com
License: GPL2
*/

//[ffytfeeds]
function ffytfeeds_func( $atts ){
    extract(shortcode_atts(array(
        'authors' => "", // comma seperated list of YT author names
        'count' => 5, // max number of videos from each source
        'width' => "640", // width in pixels of the videos to display
        'showtitles' => "false" // whether to display a title line above each video
    ), $atts));

    // if $authors is unset, return blank
    if ($authors == "") return "";
    
    // fetch feeds
    $videos = array();
    foreach(explode(",",$authors) as $author) {
        $author = trim($author);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("GData-Version: 2", "Cache-Control: no-cache"));
        curl_setopt($ch, CURLOPT_URL, "http://gdata.youtube.com/feeds/api/videos?max-results=".$count."&alt=json&orderby=published&format=5&author=".$author);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        curl_close($ch);
        
        $json = json_decode($response, true);
        if (!is_array($json)) return $response;
        
        foreach($json['feed']['entry'] as $video) {
            $date = new DateTime($video['published']['$t'], new DateTimeZone('Zulu'));
            $date->setTimezone(new DateTimeZone('Australia/Brisbane'));
            $videos[] = array(
                'author' => $author,
                'date' => $date,
                'title' => $video['title']['$t'],
                'url' => $video['content']['url'],
                'embed' => '<iframe width="'.$width.'" height="'.
                    ($video['media$group']['yt$aspectRatio']['$t'] == 'widescreen' ? round($width*9/16) : round($width*3/4)).
                    '" src="http://www.youtube.com/embed/'.
                    $video['media$group']['yt$videoid']['$t'].
                    '?rel=0" frameborder="0" allowfullscreen></iframe>'
            );
        }
    }
    
    // sort feeds
    function cmp($a,$b) {
        return $a['date']->diff($b['date'])->invert ? 0 : 1;
    }
    usort($videos,"cmp");
    
    // build content to return
    $content = "";
    foreach($videos as $video) {
        $content .= '<div class="fruit-yt-video" style="margin-bottom:40px">'.
            ($showtitles == 'true' ? '<strong><a href="http://www.youtube.com/'.$video['author'].'">'.$video['author'].
            '</a></strong> &mdash; <a href="'.$video['url'].'">'.$video['title'].'</a> ('.$video['date']->format('jS M, Y').')<br>':'').
            $video['embed']."</div>";
    }
    return $content;
}
add_shortcode( 'ffytfeeds', 'ffytfeeds_func' );

//[ffrssfeeds]
function ffrssfeeds_func( $atts ){
    define("FF_EXCERPT_LENGTH", 300);
    
    extract(shortcode_atts(array(
        'urls' => "", // comma seperated list of rss feed URLs
        'count' => 5 // max number of stories from each feed
    ), $atts));

    // if $feeds is unset, return blank
    if ($urls == "") return "";
    
    // fetch stories from each feed
    $stories = array();
    foreach(explode(",",$urls) as $url) {
        $url = htmlspecialchars_decode(trim($url));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        $response = curl_exec($ch);
        $headers = curl_getinfo($ch, CURLINFO_HEADER_OUT);
        curl_close($ch);

        $xml = simplexml_load_string($response);
        $i = 0;
        for($i = 0; $i < $count && $i < count($xml->channel->item); $i++) {
            $story = array();
            $story['feedTitle'] = (String)$xml->channel->title;
            $story['feedLink'] = (String)$xml->channel->link;
            $story['title'] = (String)$xml->channel->item[$i]->title;
            $story['link'] = (String)$xml->channel->item[$i]->link;
            $story['description'] = strlen((String)$xml->channel->item[$i]->description) > FF_EXCERPT_LENGTH + 3 ? substr((String)$xml->channel->item[$i]->description,0,500).'...' : (String)$xml->channel->item[$i]->description;
            $story['date'] = new DateTime($xml->channel->item[$i]->pubDate);
            $story['date']->setTimezone(new DateTimeZone('Australia/Brisbane'));
            $story['comments'] = (String)$xml->channel->item[$i]->comments;
            $stories[] = $story;
        }
    }
    
    // sort feeds
    function cmp($a,$b) {
        return $a['date']->diff($b['date'])->invert ? 0 : 1;
    }
    usort($stories,"cmp");
    
    // build content to return
    $content = "";
    foreach($stories as $story) {
        $content .= '<div class="fruit-rss-story" style="margin-bottom:40px">'.
            '<h2 style="display:inline"><a href="'.$story['link'].'">'.$story['title'].
            '</a></h2><span> &mdash; <a href="'.$story['feedLink'].'">'.$story['feedTitle'].'</a> ('.$story['date']->format('jS M, Y').')</span>'.
            '<p>'.$story['description']."</p></div>";
    }
    return $content;
}
add_shortcode( 'ffrssfeeds', 'ffrssfeeds_func' );

//[ffqldstategovfeed]
function ffqldstategovfeed_func( $atts ){
    define("FF_EXCERPT_LENGTH", 300);
    
    extract(shortcode_atts(array(
        'url' => "", // feed URL
    ), $atts));

    // if $feeds is unset, return blank
    if ($url == "") return "";
    
    // fetch stories from feed
    $stories = array();
    
    $url = htmlspecialchars_decode(trim($url));

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLINFO_HEADER_OUT, true);
    $response = curl_exec($ch);
    $headers = curl_getinfo($ch, CURLINFO_HEADER_OUT);
    curl_close($ch);

    $xml = simplexml_load_string($response);
    foreach($xml->entry as $item) {
        $story = array();
        $story['feedTitle'] = (String)$xml->title;
        $story['feedLink'] = (String)$xml->link['href'];
        $story['title'] = (String)$item->title;
        $story['link'] = (String)$item->link['href'];
        $story['description'] = strlen((String)$item->content) > FF_EXCERPT_LENGTH + 3 ? substr((String)$item->content,0,500).'...' : (String)$item->content;
        $story['date'] = new DateTime($item->published);
        $story['date']->setTimezone(new DateTimeZone('Australia/Brisbane'));
        $stories[] = $story;
    }
    
//    return print_r($stories,true);
    
    // sort feeds
    function cmp($a,$b) {
        return $a['date']->diff($b['date'])->invert ? 0 : 1;
    }
    usort($stories,"cmp");
    
    // build content to return
    $content = "";
    foreach($stories as $story) {
        $content .= '<div class="fruit-rss-story" style="margin-bottom:40px">'.
            '<h2 style="display:inline"><a href="'.$story['link'].'">'.$story['title'].
            '</a></h2><span> &mdash; <a href="'.$story['feedLink'].'">'.$story['feedTitle'].'</a> ('.$story['date']->format('jS M, Y').')</span>'.
            '<p>'.$story['description']."</p></div>";
    }
    return $content;
}
add_shortcode( 'ffqldstategovfeed', 'ffqldstategovfeed_func' );
